@@ -321,7 +321,7 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len, device_table *s
                                                           (apr_size_t*)len);
 
     if (convert_routine[spec->html_spec_type].converter) {
-      if (tmp)
+      if (tmp) {
         dst = convert_routine[spec->html_spec_type].converter(r, 
                                                               spec, 
                                                               tmp, 
@@ -329,7 +329,8 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len, device_table *s
                                                               len, 
                                                               entryp, 
                                                               cookie);
-      else
+      }
+      else {
         dst = convert_routine[spec->html_spec_type].converter(r,
                                                               spec, 
                                                               tmp, 
@@ -337,6 +338,7 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len, device_table *s
                                                               len, 
                                                               entryp, 
                                                               cookie);
+      }
     }
     DBG(r, "end do convert");
   }
@@ -351,7 +353,7 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len, device_table *s
   }
   ap_set_content_length(r, *len);
 
-  DBG(r, "end of chxj_exchange()");
+  DBG(r, "end of chxj_exchange() [%s]", dst);
 
   return dst;
 }
@@ -711,7 +713,6 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
   apr_status_t        rv;
   apr_bucket*         b;
   const char*         data;
-  char*               contentLength;
   char*               user_agent = NULL;
   apr_size_t          len;
   mod_chxj_ctx*       ctx = (mod_chxj_ctx *)f->ctx;
@@ -774,6 +775,11 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
           break;
         }
       }
+      if (apr_table_get(r->headers_out, "Location") || apr_table_get(r->err_headers_out, "Location")) {
+        if (r->status < HTTP_MULTIPLE_CHOICES || r->status > HTTP_TEMPORARY_REDIRECT) {
+          r->status = HTTP_MOVED_TEMPORARILY;
+        }
+      }
       ap_pass_brigade(f->next, bb);
       return APR_SUCCESS;
     }
@@ -939,12 +945,18 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
           }
         }
 
-        contentLength = apr_psprintf(pool, "%d", (int)ctx->len);
-        apr_table_setn(r->headers_out, "Content-Length", contentLength);
+        apr_table_unset(r->headers_out, "Content-Length");
+        apr_table_unset(r->err_headers_out, "Content-Length");
+        ap_set_content_length(r, (apr_off_t)ctx->len);
         
         if (ctx->len > 0) {
           DBG(r, "call pass_data_to_filter()");
           s_add_cookie_id_if_has_location_header(r, cookie);
+          if (apr_table_get(r->headers_out, "Location") || apr_table_get(r->err_headers_out, "Location")) {
+            if (r->status < HTTP_MULTIPLE_CHOICES || r->status > HTTP_TEMPORARY_REDIRECT) {
+              r->status = HTTP_MOVED_TEMPORARILY;
+            }
+          }
           rv = pass_data_to_filter(f, 
                                    (const char*)ctx->buffer, 
                                    (apr_size_t)ctx->len);
@@ -974,6 +986,11 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
             break;
 
           default:
+            if (apr_table_get(r->headers_out, "Location") || apr_table_get(r->err_headers_out, "Location")) {
+              if (r->status < HTTP_MULTIPLE_CHOICES || r->status > HTTP_TEMPORARY_REDIRECT) {
+                r->status = HTTP_MOVED_TEMPORARILY;
+              }
+            }
             break;
           }
         }
@@ -998,13 +1015,21 @@ static void
 s_add_cookie_id_if_has_location_header(request_rec *r, cookie_t *cookie)
 {
   char *location_header = (char*)apr_table_get(r->headers_out, "Location");
+  if (! location_header) {
+    location_header = (char *)apr_table_get(r->err_headers_out, "Location");
+  }
   if (cookie && location_header) {
     DBG(r, "Location Header=[%s]", location_header);
     location_header = chxj_add_cookie_parameter(r,
                                                 location_header,
                                                 cookie);
-    apr_table_setn(r->headers_out, "Location", location_header);
+    apr_table_unset(r->headers_out, "Location");
+    apr_table_add(r->headers_out, apr_pstrdup(r->pool, "Location"), location_header);
     DBG(r, "Location Header=[%s]", location_header);
+    DBG(r, "Status-Code:[%d]", r->status);
+    if (r->status < HTTP_MULTIPLE_CHOICES || r->status > HTTP_TEMPORARY_REDIRECT) {
+      r->status = HTTP_MOVED_TEMPORARILY;
+    }
   }
 }
 
@@ -1084,9 +1109,11 @@ chxj_input_handler(request_rec *r)
 
   char *chunked;
   if ((chunked = (char *)apr_table_get(r->headers_out, "Transfer-Encoding")) != NULL) {
-    if (strcasecmp(chunked, "chunked") == 0) {
+    if (strncasecmp(chunked, "chunked", sizeof("chunked")-1) == 0) {
       r->chunked = 1;  
+      apr_table_unset(r->headers_out, "Transfer-Encoding");
     }
+    DBG(r, "TRANSFER-ENCODING:[%s]",chunked);
   }
   {
     apr_pool_t *wpool;
@@ -1097,7 +1124,7 @@ chxj_input_handler(request_rec *r)
     conn_rec *c = r->connection;
     
     bb = apr_brigade_create(wpool, c->bucket_alloc);
-    e = apr_bucket_heap_create(response, res_len, NULL, c->bucket_alloc);
+    e  = apr_bucket_transient_create(response, res_len, c->bucket_alloc);
     APR_BRIGADE_INSERT_TAIL(bb, e);
     e = apr_bucket_eos_create(c->bucket_alloc);
     APR_BRIGADE_INSERT_TAIL(bb, e);
@@ -1297,7 +1324,7 @@ chxj_register_hooks(apr_pool_t* UNUSED(p))
   ap_hook_handler(chxj_qr_code_handler, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_handler(chxj_input_handler, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_translate_name(chxj_translate_name, NULL, NULL, APR_HOOK_MIDDLE);
-  ap_hook_fixups(chxj_headers_fixup, NULL, NULL, APR_HOOK_FIRST);
+  ap_hook_fixups(chxj_headers_fixup, NULL, NULL, APR_HOOK_LAST);
 }
 
 
